/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>

#ifndef _MCRYPT_H
#define _MCRYPT_H
#include <mcrypt.h>
#endif

#include "common.h"
#include "conf.h"
#include "thread.h"
#include "log.h"
#include "bignum.h"
#include "crypt.h"
#include "packet.h"
#include "node.h"
#include "caller.h"


caller_t *caller_create(int sockd) {
	caller_t *caller;

	if ((caller = malloc(sizeof(caller_t))) == NULL) {
		ERROR("could not malloc space for a new caller");
		return NULL;
	}
	bzero(caller,sizeof(caller_t));
	
	object_init(caller,caller_free,caller);
	thread_init(caller,caller_run,caller);

	caller->proxy_sockd	= sockd;
	caller->max_hopid	= -1;

	return caller;
} /* end node_create */


void caller_free(void *caller) {

	thread_destroy(caller);
	object_destroy(caller);

	free(caller);
	
	return; /* void */
} /* end caller_free */


void caller_run(void *caller) {
	int i,b,bytes;
	unsigned char response[1 + 16];
	unsigned char *buf;
	unsigned char packet[PACKET_SIZE] = {0};
	char *positions[] = {
		"first",
		"second",
		"third",
		"fourth",
		"fifth"
	};

	bzero(response,sizeof(response));


	if (caller_proxy_initiate((caller_t *)caller) == false)
		goto caller_finish;


	for (i = 0; i < CALLER_MAX_NODES; i++) {
		get_node(&((caller_t *)caller)->nodes[i]);
		((caller_t *)caller)->nodes[i].hopid = i;
	}

	for (i = 0; i < CALLER_MAX_NODES; i++) {
		DEBUG("attempting to build %s tunnel with node %s",positions[i],((caller_t *)caller)->nodes[i].host);

		if (i == 0) {
			if (caller_link_connect((caller_t *)caller,&((caller_t *)caller)->nodes[0]) == false) {
				WARN("failed to setup link to %s node",positions[i]);
				goto caller_finish;
			}
		} else {
			if (caller_passthru_connect((caller_t *)caller,&((caller_t *)caller)->nodes[0]) == false) {
				WARN("failed to setup link to %s node",positions[i]);
				goto caller_finish;
			}
		}

		if (caller_node_connect((caller_t *)caller,&((caller_t *)caller)->nodes[i]) == false) {
			WARN("could not connect to node %s:%i",((caller_t *)caller)->nodes[i].host,((caller_t *)caller)->nodes[i].port);
			goto caller_finish;
		} else {
			DEBUG("%s tunnel setup with node %s",positions[i],((caller_t *)caller)->nodes[i].host);
		}
	}

	/* notify proxy */
	FLAG_SET(response[0],0x01); /* IPv4 */
	memcpy(&response[1],&((caller_t *)caller)->dest_sin.sin_addr,4);

	/* just return the response w/ all nuls; 0 flag is failure */
	bytes	= sizeof(response);
	buf	= response;

	do {
		if ((b = write(((caller_t *)caller)->proxy_sockd,buf,bytes)) < 0) {
			WARN("socket write failed: %s",strerror(errno));
			goto caller_finish;
		}
		bytes	-= b;
		buf	+= b;
	} while (bytes > 0);


	/* start proxying loop */


	usleep(500);

	for (i = (CALLER_MAX_NODES-1); i >= 0; i--) {
		ERROR("tearing down %s tunnel with node %s",positions[i],((caller_t *)caller)->nodes[i].host);

		CALLER_SET_FLAGS(((caller_t *)caller)->max_hopid,PACKET_NODE_FWRD,packet);
		FLAG_SET(packet[PACKET_FLAG_POS(i)],PACKET_NODE_KILL);

		CALLER_WRAP_PACKET(*((caller_t *)caller),((caller_t *)caller)->max_hopid,packet);

		if (write_packet(((caller_t *)caller)->route_sockd,packet) != PACKET_SIZE) {
			WARN("failed to tear down %s tunnel with node %s",positions[i],((caller_t *)caller)->nodes[i].host);
			goto caller_finish;
		}
	}



	caller_finish:

	INFO("caller finishing");
	close(((caller_t *)caller)->route_sockd); /* how to check if it is connected? */
	return; /* void */
	
} /* caller_run */


bool caller_proxy_initiate(caller_t *caller) {
	int b, bytes;
	struct hostent *hostent;
	unsigned char request[1 + 256 + 2];
	unsigned char response[1 + 16];
	unsigned char *buf;
	char host[257];


	bzero(request,sizeof(request));
	bzero(response,sizeof(response));
	bzero(host,sizeof(host));


	/*
	 * read request from proxy
	 *
	 * 0x01 IPV4
	 * 0x02 IPV6
	 * 0x04 NAME
	 *
	 */
	bytes	= sizeof(request);
	buf	= request;

	do {
		if ((b = read(((caller_t *)caller)->proxy_sockd,buf,bytes)) < 1) {
			switch (b) {
				case 0:
					WARN("premature EOF on socket");
					goto fail;
				default:
					ERROR("socket read failed: %s",strerror(errno));
					goto fail;
			}
		}
		bytes	-= b;
		buf	+= b;
	} while (bytes > 0);


	if (FLAG_ISSET(request[0],0x01)) {
		caller->dest_sin.sin_family = AF_INET;
		memcpy(&caller->dest_sin.sin_addr,&request[1],4);
		memcpy(&caller->dest_sin.sin_port,&request[257],2);
		return true;
	} else if (FLAG_ISSET(request[0],0x02)) {
		WARN("request to proxy for IPv6 address: IPv6 not supported");	
		goto fail;
	} else if (FLAG_ISSET(request[0],0x04)) {
		memcpy(host,&request[1],256);
		memcpy(&caller->dest_sin.sin_port,&request[257],2);

		if ((hostent = gethostbyname(host)) == NULL) {
			WARN("failed to resolve host name in proxy request");
			goto fail;
		}
		
		if (hostent->h_addrtype == AF_INET6) {
			WARN("hostname resolved to an IPv6 address: IPv6 not supported");
			goto fail;
		}

		if (hostent->h_addr_list[0] != NULL) {
			DEBUG("length of h_addr's: %i",hostent->h_length);
			memcpy(&caller->dest_sin.sin_addr,hostent->h_addr_list[0],4);
			return true;
		} else {
			goto fail;
		}
		
	} else {
		WARN("protocol violation in proxy connection request");
		goto fail;
	}


	fail:

	/* just return the response w/ all nuls; 0 flag is failure */
	bytes	= sizeof(response);
	buf	= response;

	do {
		if ((b = write(((caller_t *)caller)->proxy_sockd,buf,bytes)) < 0) {
			WARN("socket write failed: %s",strerror(errno));
			return false;
		}
		bytes	-= b;
		buf	+= b;
	} while (bytes > 0);
	
	return false;

} /* end caller_proxy_initiate */


bool caller_link_connect(caller_t *caller,node_info_t *node) {
	unsigned char packet[PACKET_SIZE];
	
	FLAG_SET(packet[0],PACKET_LINK_INIT);
	packet[1]	= 0; /* caller only shares a link w/ the 1st node */
	packet[2]	= PROTOCOL_MAJOR;
	packet[3]	= PROTOCOL_MINOR;
	packet[4]	= PROTOCOL_PATCH;

	if ((caller->route_sockd = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP)) < 0) {
		ERROR("could not create socket: %s",strerror(errno));
		return false;
	}

	caller->route_sin.sin_family	= AF_INET;
	caller->route_sin.sin_port	= htons(node->port);

	switch (inet_pton(AF_INET,node->host,&caller->route_sin.sin_addr)) {
		case 0:
			ERROR("could not convert address %s",node->host);
			return false;
		case -1:
			ERROR("could not convert address %s: %s",node->host,strerror(errno));
			return false;
	}

	if (connect(caller->route_sockd, (struct sockaddr *)&caller->route_sin,(socklen_t)sizeof(struct sockaddr_in)) != 0) {
		ERROR("could not connect to %s:%i: %s",node->host,node->port,strerror(errno));
		return false;
	}	

	if (write_packet(caller->route_sockd,packet) != PACKET_SIZE) {
		WARN("could not write packet during link initialization");
		return false;
	}

	DEBUG("link setup to %s:%i",node->host,node->port);
	
	return true;

} /* end caller_link_connect */


bool caller_passthru_connect(caller_t *caller,node_info_t *node) {
	unsigned char request[1 + 256 + 2];
	unsigned char response[1 + 16];
	unsigned char packet[PACKET_SIZE];
	unsigned char *buf;
	int bytes,r;

	bzero(request,sizeof(request));

	FLAG_SET(request[0],PACKET_CNCT_REQ_IPV4);
	
	if (inet_pton(AF_INET,node->host,&request[1]) <= 0) {
		WARN("could not connect to node: inet_pton failed");
		return false;
	}

	*((uint16_t *)(&request[257])) = htons(node->port);

	bytes	= sizeof(request);
	buf	= request;


	FLAG_CLR(packet[0]);
	FLAG_SET(packet[0],PACKET_LINK_NBND);

	DEBUG("writing connection request to node");

	do {
		CALLER_CLR_FLAGS(caller->max_hopid,packet);
		CALLER_SET_FLAGS((caller->max_hopid-1),PACKET_NODE_FWRD,packet);
		FLAG_SET(packet[PACKET_FLAG_POS(caller->max_hopid)],PACKET_NODE_CNCT);

		memcpy(&packet[PACKET_DATA_POS(caller->max_hopid)],buf,MIN(bytes,PACKET_DATA_SIZE));
		packet[PACKET_PLEN_POS(caller->max_hopid)] = MIN(bytes,PACKET_DATA_SIZE);

		CALLER_WRAP_PACKET(*caller,caller->max_hopid,packet);

		if (write_packet(caller->route_sockd,packet) != PACKET_SIZE) {
			WARN("unable to write packet");
			return false;
		}
		
		bytes	-= PACKET_DATA_SIZE;
		buf	+= PACKET_DATA_SIZE;
	
	} while (bytes > 0);


	bytes	= 17;
	buf	= response;
	r	= -1;

	DEBUG("reading connection response from node");

	do {

		if (read_packet(caller->route_sockd,packet) != PACKET_SIZE) {
			WARN("unable to read packet");
			return false;
		}

		CALLER_UNWRAP_PACKET(*caller,caller->max_hopid,packet,r);
	
		if (r != -1) {
			WARN("failed to unwrap packet; bad packet from node %i",r);
			return false;
		} else if (FLAG_ISSET(packet[0],PACKET_LINK_NBND)) {

			memcpy(buf,&packet[PACKET_DATA_POS(caller->max_hopid)],MIN(bytes,PACKET_DATA_SIZE));
			buf	+= PACKET_DATA_SIZE;
			bytes	-= PACKET_DATA_SIZE;

		} else if (packet[0] == PACKET_LINK_NULL) {
			continue;
		} else if (packet[0] == PACKET_LINK_KILL) {
			WARN("got kill request on link");
			return false;
		} else {
			WARN("protocol violation: bad packet");
			return false;
		}
	
	} while (bytes > 0);


	return (response[0] != PACKET_CNCT_RSP_FAIL);

} /* end caller_passthru_connect */


bool caller_node_connect(caller_t *caller,node_info_t *node) {
	int i, b, bytes, z_size, _cipher;
	dh_t dh;
	mpz_t n;
	bool rval;
	unsigned char *buf;
	unsigned char packet[PACKET_SIZE];
	unsigned char input[NODE_CALLER_INITIATE_INPUT_SIZE];
	unsigned char *cipher		= &input[0];
	unsigned char *hash		= &input[1];
	unsigned char *p_size		= &input[2];
	unsigned char *g_size		= &input[3];
	unsigned char *y_size		= &input[4];
	unsigned char *prime		= &input[5];
	unsigned char *generator	= &prime[NODE_MAX_P_SIZE];
	unsigned char *pubk		= &generator[NODE_MAX_G_SIZE];
	unsigned char *seed		= &pubk[NODE_MAX_Y_SIZE];
	unsigned char key_material[NODE_KEYMATERIAL_SIZE];
        unsigned char *encrypt_key      = &key_material[0];
	unsigned char *decrypt_key      = &encrypt_key[16];
	unsigned char *cipher_iv        = &decrypt_key[16];
	unsigned char *hmac_key         = &cipher_iv[16];
	uint16_t *seq_init              = (uint16_t *)&hmac_key[64];
	unsigned char secret[NODE_MAX_P_SIZE];
	unsigned char _seed[NODE_SEED_SIZE];


	/*
	 * we don't want to leak (stale) sensitive data in
	 * memory or on the stack; automatic variables
	 * aren't initialized.
	 */
	bzero(input,sizeof(input));
	bzero(packet,sizeof(packet));

	dh_init(&dh);
	mpz_init(n);
	rval = false;


	*cipher	= _cipher = NODE_CIPHER_RIJNDAEL;
	*hash	= NODE_HASH_TIGER;


	dh_generate_parameters(&dh,256,3);
	dh_generate_key(&dh);

	*p_size	= (bignum_format_u(dh.p,prime) - 1);
	*g_size	= (bignum_format_u(dh.g,generator) - 1);
	*y_size	= (bignum_format_u(dh.y,pubk) - 1);
	rand_get(rand_getctx(),seed,NODE_SEED_SIZE);
	memmove(_seed,seed,NODE_SEED_SIZE);

	if (ENABLE_DEBUG) {
		fprintf(stdout,"Caller's prime:\n");
		PRINT_BYTES(stdout,prime,(*p_size + 1));

		fprintf(stdout,"Caller's generator:\n");
		PRINT_BYTES(stdout,generator,(*g_size + 1));

		fprintf(stdout,"Caller's pubk:\n");
		PRINT_BYTES(stdout,pubk,(*y_size + 1));
	} /* DEBUG */


	buf	= input;
	bytes	= NODE_CALLER_INITIATE_INPUT_SIZE;

	FLAG_CLR(packet[0]);
	FLAG_SET(packet[0],PACKET_LINK_NBND);

	do {
	
		CALLER_CLR_FLAGS(caller->max_hopid,packet);
		CALLER_SET_FLAGS(caller->max_hopid,PACKET_NODE_FWRD,packet);

		FLAG_SET(packet[PACKET_FLAG_POS(node->hopid)],PACKET_NODE_INIT);
		memcpy(&packet[PACKET_DATA_POS(node->hopid)],buf,MIN(bytes,PACKET_DATA_SIZE));

		CALLER_WRAP_PACKET(*caller,caller->max_hopid,packet);

		if (write_packet(caller->route_sockd,packet) != PACKET_SIZE) {
			WARN("failed to write packet during node initialization");
			goto caller_connect_finish;			
		}
	
		bytes	-= PACKET_DATA_SIZE;
		buf	+= PACKET_DATA_SIZE;
	
	} while (bytes > 0);	


	/* sent everything. read in nodes pubk and re-use our buffer */
	
	buf	= input;
	bytes	= 1 + NODE_MAX_Y_SIZE;
	b	= -1;

	do {

		if (read_packet(caller->route_sockd,packet) != PACKET_SIZE) {
			WARN("failed to read packet during node initialization");
			goto caller_connect_finish;
		}

		CALLER_UNWRAP_PACKET(*caller,caller->max_hopid,packet,b);

		if (b != -1) {
			WARN("failed to unwrap packet: bad packet from node %i",b);
			goto caller_connect_finish;
		}

		memcpy(buf,&packet[PACKET_DATA_POS(node->hopid)],PACKET_DATA_SIZE);
		buf	+= PACKET_DATA_SIZE;
		bytes	-= PACKET_DATA_SIZE;
	} while (bytes > 0);


	if (ENABLE_DEBUG) {
		printf("Received node's pubk:\n");
		PRINT_BYTES(stdout,&input[1],(input[0] + 1));
	} /* DEBUG */

	bignum_parse_u(n,(input[0] + 1),&input[1]);

	dh_compute_key(&dh,n);

	z_size = bignum_format_u(dh.z,secret);
	
	if (ENABLE_DEBUG) {
		printf("Caller's shared secret:\n");
		PRINT_BYTES(stdout,secret,z_size);
	} /* DEBUG */

	prf_expand(node->hash_type,key_material,sizeof(key_material),secret,z_size,_seed,NODE_SEED_SIZE);

	/* 
	 * char host[256];
	 * uint16_t port;
	 * int hopid;
	 * int seq;
	 * MCRYPT encrypt_td;
	 * MCRYPT decrypt_td;
	 * int hash_type;
	 * unsigned char hmac_key[64];
	 */


	/* setup ciphers */
	/*switch(_cipher) {
		case NODE_CIPHER_AES:
			if ((node->encrypt_td = mcrypt_module_open(MCRYPT_RIJNDAEL_128,NULL,MCRYPT_CBC,NULL)) == MCRYPT_FAILED) {
				ERROR("failed to open mcrypt cipher %s",MCRYPT_RIJNDAEL_128);
				rval = false;
			}
			if ((node->decrypt_td = mcrypt_module_open(MCRYPT_RIJNDAEL_128,NULL,MCRYPT_CBC,NULL)) == MCRYPT_FAILED) {
				ERROR("failed to open mcrypt cipher %s",MCRYPT_RIJNDAEL_128);
				rval = false;
			}
			break;
		case NODE_CIPHER_TWOFISH:
			if ((node->encrypt_td = mcrypt_module_open(MCRYPT_TWOFISH,NULL,MCRYPT_CBC,NULL)) == MCRYPT_FAILED) {
				ERROR("failed to open mcrypt cipher %s",MCRYPT_TWOFISH);
				rval = false;
			}
			if ((node->decrypt_td = mcrypt_module_open(MCRYPT_RIJNDAEL_128,NULL,MCRYPT_CBC,NULL)) == MCRYPT_FAILED) {
				ERROR("failed to open mcrypt cipher %s",MCRYPT_RIJNDAEL_128);
				rval = false;
			}
			break;
		default:
			WARN("bad cipher requested in caller initiation: %i",*cipher);
			rval = false;
	}
	mcrypt_generic_init(node->encrypt_td,encrypt_key,16,cipher_iv);
	mcrypt_generic_init(node->decrypt_td,decrypt_key,16,cipher_iv);*/

	cipher_init(&node->encrypt_cipher,_cipher,encrypt_key,cipher_iv);
	cipher_init(&node->decrypt_cipher,_cipher,decrypt_key,cipher_iv);

	/* end cipher setup */

	memcpy(node->hmac_key,hmac_key,64);
	node->seq = ntohs(*seq_init);

	if (ENABLE_DEBUG) {
		printf("Caller's encrypt key:\n");
		PRINT_BYTES(stdout,encrypt_key,16);

		printf("Caller's decrypt key:\n");
		PRINT_BYTES(stdout,decrypt_key,16);

		printf("Caller's hmac key:\n");
		PRINT_BYTES(stdout,node->hmac_key,64);
	}


	caller->max_hopid++;
	rval = true;

	caller_connect_finish:

	mpz_clear(n);
	dh_destroy(&dh);
	return rval;

} /* end caller_connect */


void get_node(node_info_t *info) {
	extern conf_t conf;

	object_lock(&conf);

	strcpy(info->host,"127.0.0.1");
	info->port = (uint16_t)conf.port;
	info->hash_type = TIGER_192;

	object_unlock(&conf);

	return; /* void */

} /* end get_node */

