/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#ifndef _CALLER_H
#define _CALLER_H 1

#include <netinet/in.h>

#ifndef _MCRYPT_H
#define _MCRYPT_H
#include <mcrypt.h>
#endif

#include "common.h"
#include "thread.h"
#include "crypt.h"

#define CALLER_MAX_NODES	3 /* 5 gives too much debug info */

/* CALLER_WRAP_PACKET(caller_t,int,unsigned char *) */
#define CALLER_WRAP_PACKET(__caller,__max_hopid,__packet) do { \
	int i; \
	unsigned char hmac[64]; \
	for (i = (__max_hopid); i >= 0 ; i--) { \
		HMAC((__caller).nodes[i].hash_type,hmac,(__caller).nodes[i].hmac_key,sizeof((__caller).nodes[i].hmac_key),&(__packet)[PACKET_HASH_BEG(i)],PACKET_HASH_LEN(i)); \
		memmove(&(__packet)[PACKET_MAC_POS(i)],hmac,10); \
		cipher_encrypt(&(__caller).nodes[i].encrypt_cipher,&(__packet)[PACKET_ENC_BEG(i)],PACKET_ENC_LEN(i)); \
	} \
} while (0)

#define CALLER_UNWRAP_PACKET(__caller,__max_hopid,__packet,__return) do { \
	int i; \
	unsigned char hmac[64]; \
	for (i = 0; i <= (__max_hopid); i++) { \
		cipher_decrypt(&(__caller).nodes[i].decrypt_cipher,&(__packet)[PACKET_ENC_BEG(i)],PACKET_ENC_LEN(i)); \
		HMAC((__caller).nodes[i].hash_type,hmac,(__caller).nodes[i].hmac_key,sizeof((__caller).nodes[i].hmac_key),&(__packet)[PACKET_HASH_BEG(i)],PACKET_HASH_LEN(i)); \
		if (memcmp(hmac,&(__packet)[PACKET_MAC_POS(i)],10) != 0) { \
			WARN("Bad MAC in packet."); \
			(__return) = i; \
			break; \
		} \
	} \
} while (0)

#define CALLER_SET_FLAGS(__max_hopid,__flag,__packet) do { \
	int i; \
	for (i = 0; i <= (__max_hopid); i++) \
		FLAG_SET((__packet)[PACKET_FLAG_POS(i)],(__flag)); \
} while (0)

#define CALLER_CLR_FLAGS(__max_hopid,__packet) do { \
	int i; \
	for (i = 0; i <= (__max_hopid); i++) \
		FLAG_CLR((__packet)[PACKET_FLAG_POS(i)]); \
} while (0)

#define CALLER_ISSET_FLAGS(__max_hopid,__flag,__packet,__return) do { \
	int i; \
	(__return) == true; \
	for (i = 0; i <= (__max_hopid); i++) { \
		if (!FLAG_ISSET((__packet)[PACKET_FLAG_POS(i)],(__flag))) { \
			(__return) = false; \
			break; \
		} \
	} \
} while (0)


typedef struct node_info {
	char host[256];
	uint16_t port;

	int hopid;
	int seq;
	//MCRYPT encrypt_td;
	//MCRYPT decrypt_td;
	cipher_t encrypt_cipher;
	cipher_t decrypt_cipher;
	int hash_type;
	unsigned char hmac_key[64];
} node_info_t;


typedef struct caller {
	thread_t thread; /* inherit thread & object */

	int proxy_sockd;
	struct sockaddr_in dest_sin;

	int route_sockd;
	struct sockaddr_in route_sin;

	node_info_t nodes[5];
	int max_hopid;

} caller_t;

caller_t *caller_create(int);
void caller_free(void *);

void caller_run(void *);

bool caller_proxy_initiate(caller_t *);
bool caller_link_connect(caller_t *,node_info_t *);
bool caller_passthru_connect(caller_t *,node_info_t *);
bool caller_node_connect(caller_t *,node_info_t *);

void get_node(node_info_t *);


#endif /* _CALLER_H */
