/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "common.h"
#include "options.h"
#include "conf.h"
#include "log.h"
#include "crypt.h"
#include "thread.h"
#include "parsecfg.h"


void conf_init() {
	extern conf_t conf;
	
	object_init(&conf,NULL,NULL);

	/* string defaults; for leak detection in conf_parse */
	conf.__random		= DEFAULT_RANDOM;
	conf.__user		= NULL;
	conf.__group		= NULL;

	/* defaults */
	conf.daemon		= true;
	conf.port		= DEFAULT_PORT;
	conf.verbosity		= DEFAULT_VERBOSITY;
	conf.random		= conf.__random;
	conf.user		= conf.__user;
	conf.group		= conf.__group;
	conf.enable_error	= false;
	conf.enable_warn	= false;
	conf.enable_info	= false;
	conf.enable_debug	= false;
}

void conf_destroy() {
	extern conf_t conf;

	object_destroy(&conf);

}


bool conf_parse() {
	extern options_t options;
	extern conf_t conf;
	unsigned int port = 0;
	bool daemon = false;
	char *random;

	object_lock(&conf);


	/* make sure we don't leak memory on multiple parses */
	if (conf.random != NULL	&& conf.random != conf.__random)
		free(conf.random);
	if (conf.user != NULL	&& conf.user != conf.__user)
		free(conf.user);
	if (conf.group != NULL	&& conf.group != conf.__group)
		free(conf.group);

	
	INFO("Reading configuration from %s",(options.config == NULL)? DEFAULT_CONFIG : options.config);

	if (options.foreground) {
		conf.daemon = false;
	} else if (options.daemon) {
		conf.daemon = true;
	} else {
		if (fetchVarFromCfgFile((options.config == NULL)? DEFAULT_CONFIG : options.config,"daemon",&daemon,CFG_BOOL,CFG_SIMPLE,0,NULL) == 0) {
			conf.daemon = daemon;
		} else {
			WARN("Defaulting to %s.",(conf.daemon)? "run as a daemon": "run in the foreground");
		}
	}

	if (options.port != 0) {
		conf.port = options.port;
	} else {
		if (fetchVarFromCfgFile((options.config == NULL)? DEFAULT_CONFIG : options.config,"port",&port,CFG_UINT,CFG_SIMPLE,0,NULL) == 0) {
			if (0 < port && port < 65536) {
				conf.port = (unsigned short) port;
			} else {
				ERROR("Illegal port found in configuration file: %i",port);
				return false;
			}
		} else {
			WARN("Using default port %i",conf.port);
		}
	}


	if (options.random != NULL) {
		conf.random = options.random;
	} else {
		if (fetchVarFromCfgFile((options.config == NULL)? DEFAULT_CONFIG : options.config,"random",&random,CFG_STRING,CFG_SIMPLE,0,NULL) == 0) {
			conf.random = strdup(random);
		} else {
			WARN("Defaulting to random device %s.",conf.random);
		}
	
	}


	conf.verbosity = MIN(4,(options.verbosity == 0)? DEFAULT_VERBOSITY : options.verbosity);
	
	switch (conf.verbosity) {
		case 4:
			conf.enable_debug	= true;
		case 3:
			conf.enable_info	= true;
		case 2:
			conf.enable_warn	= true;
		case 1:
		default:
			conf.enable_error	= true;
	}

	


	if (options.user != NULL) {
		conf.user = (char *)strdup(options.user);
	}
	
	if (options.group != NULL) {
		conf.group = (char *)strdup(options.group);
	}

	object_unlock(&conf);
		
	return true;
}

