/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
/*#include <openssl/bn.h>
#include <openssl/dh.h>
#include <openssl/hmac.h>*/

#ifndef _MCRYPT_H
#define _MCRYPT_H
#include <mcrypt.h>
#endif

#include "common.h"
#include "thread.h"
#include "log.h"
#include "packet.h"
#include "link.h"
#include "node.h"
#include "crypt.h"


void link_init(link_t *link) {

	/*if ((link->dh = DH_new()) == NULL) {
		ERROR("Could not get a new DH object.");
		return;
	}

	if ((link->td = mcrypt_module_open(MCRYPT_RIJNDAEL_128,NULL,MCRYPT_CBC,NULL)) == NULL) {
		ERROR("Could not get new Mcrypt object.");
		return;
	}

	link->in_seq	= 0;
	link->out_seq	= 0;*/

	return; /* void */
} /* end link_init */


void link_destroy(link_t *link) {


	return; /* void */

} /* end link_destroy */


link_t *link_create() {
	link_t *link;
	
	if ((link = malloc(sizeof(link_t))) == NULL) {
		ERROR("could not malloc space for a new link");
		return NULL;
	}
	bzero(link,sizeof(link_t));

	return link;
} /* end link_create */


void link_free(link_t *this) {
	link_destroy(this);
	free(this);
	
	return; /* void */
} /* end link_free */


bool link_readp(link_t *this) {
	ssize_t bytes = 0;
	size_t off    = 0;
	int i;
	unsigned char md[/*EVP_MAX_MD_SIZE*/64];
	int len;
	unsigned short int seq;

	do {
		if ((bytes = read(this->sockfd,(void *)&this->recvbuf[off],(size_t)(PACKET_SIZE-off))) <= 0) {
			if (bytes == 0) {
				WARN("premature EOF on socket");
			} else {
				ERROR("socket read failed: %s",strerror(errno));
			}
			return false;
		}
		off += bytes;
	} while (off < PACKET_SIZE);


	if (this->encrypted) {
		printf("Read Ciphertext:\n");
		PRINT_BYTES(stdout,this->recvbuf,129);
		mdecrypt_generic(this->td,&this->recvbuf[NODE_ENC_BEG(this->hopid)],NODE_ENC_LEN(this->hopid));
		printf("Read Plaintext:\n");
		PRINT_BYTES(stdout,this->recvbuf,129);


		/*HMAC(EVP_sha1(),this->hmac_key,16,(this->recvbuf+NODE_HASH_BEG(this->hopid)),NODE_HASH_LEN(this->hopid),md,&len);*/

		FOLD_SHA1_160(md);

		if (memcmp(md,(this->recvbuf+NODE_MAC_POS(this->hopid)),10) != 0) {
			WARN("received bad packet MAC");
			return false;
		}
		
	}

	if (this->recvbuf[0] != this->lflag) {
		WARN("received unexpected packet type");
		return false;
	}

	seq = ntohs(*(unsigned short int *)(this->recvbuf+NODE_SEQ_POS(this->hopid)));
	if (this->in_seq != seq) {
		WARN("received bad packet sequence; expected %i but got %i",this->in_seq,seq);
		return false;
	}

	if (this->encrypted)
		this->in_seq++;
	
	return true;
}

void link_writep(link_t *this) {
	ssize_t bytes = 0;
	size_t off    = 0;
	char *str;
	unsigned char md[/*EVP_MAX_MD_SIZE*/64];
	int len;

	*(unsigned char *)(this->recvbuf+NODE_FLAG_POS(this->hopid)) = this->nflag;

	*(unsigned short int *)(this->recvbuf+NODE_SEQ_POS(this->hopid)) = htons(this->out_seq);

	if (this->encrypted)
		this->out_seq++;

	if (this->encrypted) {
		/*HMAC(EVP_sha1(),this->hmac_key,16,(this->recvbuf+NODE_HASH_BEG(this->hopid)),NODE_HASH_LEN(this->hopid),md,&len);*/
		XOR_ARRAY((this->recvbuf+NODE_MAC_POS(this->hopid)),md,(md+10),10);

		printf("Wrote Plaintext:\n");
		PRINT_BYTES(stdout,this->recvbuf,129);
		mcrypt_generic(this->td,(this->recvbuf+NODE_ENC_BEG(this->hopid)),NODE_ENC_LEN(this->hopid));
		printf("Wrote Ciphertext:\n");
		PRINT_BYTES(stdout,this->recvbuf,129);
	}

	do {
		if ((bytes = write(this->sockfd,&this->sendbuf[off],(PACKET_SIZE-off))) < 0) {
			ERROR("failed to write packet: %s",strerror(errno));
			return;
		}
		off += bytes;
	} while (off < PACKET_SIZE);

}

bool link_read(link_t *this,void *buf,int len) {
	int i;

	for (i = PACKET_BOUNDARY(len)/PACKET_DATA_SIZE; i > 0; i--) {
		if (link_readp(this)) {
			memcpy(buf,&this->recvbuf[NODE_DATA_POS(this->hopid)],PACKET_DATA_SIZE);
			buf += PACKET_DATA_SIZE;
		} else {
			return false;
		}
	}
	return true;
}

bool link_write(link_t *this,void *buf,int len) {
	int i;

	this->sendbuf[0] = this->lflag;
	this->sendbuf[NODE_FLAG_POS(this->hopid)] = this->nflag;
	
	for (i = PACKET_BOUNDARY(len)/PACKET_DATA_SIZE; i > 0; i--) {
		memcpy(&this->sendbuf[NODE_DATA_POS(this->hopid)],buf,PACKET_DATA_SIZE);
		buf += PACKET_DATA_SIZE;
		/* RE-WRITE writep to return status */
		link_writep(this);
	}
	return true;
}
