/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <pthread.h>

#include "common.h"
#include "options.h"
#include "conf.h"
#include "log.h"


bool __log_level(int level) {
	switch (level) {
		case ANET_LOG_ERROR:
			return conf.enable_error;
		case ANET_LOG_WARN:
			return conf.enable_warn;
		case ANET_LOG_INFO:	
			return conf.enable_info;
		case ANET_LOG_DEBUG:
			return conf.enable_debug;
		default:
			return false;
	}
}


void log_init(conf_t *conf) {
	if (conf == NULL) {
		openlog(PACKAGE,LOG_CONS | LOG_PERROR | LOG_PID,LOG_DAEMON);
		return; /* void */
	}

	if (conf->daemon) {
		openlog(PACKAGE,LOG_CONS | LOG_PID,LOG_DAEMON);
		return; /* void */
	} else {
		openlog(PACKAGE,LOG_CONS | LOG_PERROR | LOG_PID,LOG_DAEMON);
		return; /* void */
	}
}


void log(int type,char *file,int line,char *format,...) {
	va_list args;
	int level;
	char str[255] = {0};

	if (ENABLE_TRACK_SOURCE && ENABLE_TRACK_THREAD) {
		snprintf(str,sizeof(str),"%s:%i:%i:",file,line,(int)pthread_self());
	} else {
		if (ENABLE_TRACK_SOURCE) {
			snprintf(str,sizeof(str),"%s:%i:",file,line);
		} else if (ENABLE_TRACK_THREAD) {
			snprintf(str,sizeof(str),"%i:",(int)pthread_self());
		}
	}

	switch(type) {
		case ANET_LOG_ERROR:
			level = LOG_ERR;
			snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","error: ");
			break;
		case ANET_LOG_WARN:
			if (ENABLE_WARN) {
				level = LOG_WARNING;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","warn: ");
				break;
			} else
				return;
		case ANET_LOG_INFO:
			if (ENABLE_INFO) {
				level = LOG_INFO;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","info: ");
				break;
			} else
				return;
		case ANET_LOG_DEBUG:
			if (ENABLE_DEBUG) {
				level = LOG_DEBUG;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","debug: ");
				break;
			} else
				return;
		default:
			syslog(LOG_ERR,"log error: invalid argument specified, %i, in file %s, line %i",type,file,line);
			level = LOG_ERR;
	}

	va_start(args,format);
	 /* decrease size argument by 1? are all vsnprintf()
	  * implementations guaranteed to NUL terminate? then
	  * we'll have to make sure str[254] is NUL as well.
	  */
	vsnprintf(&str[strlen(str)],sizeof(str) - strlen(str),format,args);
	va_end(args);

	syslog(level,str);
}
