/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */
#include <pthread.h>

#include "common.h"

#ifndef __OBJECT_H
#define __OBJECT_H


typedef void (*destructor_t)(void *);
typedef void *destructor_arg_t;
typedef struct object object_t;

typedef object_t *Object;


struct object {
	int __gc_ref_count;

	destructor_t		__free;
	destructor_arg_t	__free_arg;

	pthread_mutex_t		__sync_mutex;
	pthread_mutexattr_t	__sync_mutexattr;

	pthread_mutex_t		__gc_mutex;
	pthread_mutexattr_t	__gc_mutexattr;

	pthread_mutex_t		__cond_mutex;
	pthread_mutexattr_t	__cond_mutexattr;
	pthread_cond_t		__cond_var;
	pthread_condattr_t	__cond_attr;

	void (*lock)(void *);
	void (*unlock)(void *);

	void (*retain)(void *);
	void (*release)(void *);
	
	void (*wait)(void *);
	void (*timedwait)(void *,const struct timespec *);
	void (*signal)(void *);
	void (*broadcast)(void *);
};

void object_init(void *,void (*)(void *),void *);
void object_destroy(void *);

object_t *object_create(void);
void object_free(object_t *);
void __object_free(void *);

void object_lock(void *);
void object_unlock(void *);

void object_retain(void *);
void object_release(void *);

void object_wait(void *);
void object_timedwait(void *,const struct timespec *);
void object_signal(void *);
void object_broadcast(void *);

#endif
