/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "common.h"
#include "options.h"
#include "conf.h"
#include "log.h"
#include "thread.h"
#include "parsecfg.h"


void conf_init() {
	extern Conf conf;
	extern void conf_lock(void);
	extern void conf_unlock(void);
	
	if ((conf = malloc(sizeof(conf_t))) == NULL) {
		perror("an error occured during configuration");
		exit(EXIT_FAILURE);
	}
	memset(conf,0,sizeof(conf_t));

	if ((conf->mutex = mutex_create()) == NULL) {
		perror("an error occured during configuration");
		exit(EXIT_FAILURE);
	}

	conf->lock   = conf_lock;
	conf->unlock = conf_unlock;

	/* defaults */
	conf->daemon    = true;
	conf->port      = DEFAULT_PORT;
	conf->verbosity = DEFAULT_VERBOSITY;
	conf->user      = NULL;
	conf->group     = NULL;

}

void conf_destroy() {
	extern Conf conf;

	if (conf->mutex != NULL)
		mutex_destroy(conf->mutex);

	free(conf);
}

bool conf_parse() {
	extern Options options;
	extern Conf conf;
	unsigned int port = 0;
	bool daemon = false;

	conf->lock();
	

	if (options->foreground) {
		conf->daemon = false;
	} else if (options->daemon) {
		conf->daemon = true;
	} else {
		if (fetchVarFromCfgFile((options->config == NULL)? DEFAULT_CONFIG : options->config,"daemon",&daemon,CFG_BOOL,CFG_SIMPLE,0,NULL) == 0) {
			conf->daemon = daemon;
		} else {
			fprintf(stderr,"an error occured during configuration\n");
			return false;
		}
	}

	if (options->port != 0) {
		conf->port = options->port;
	} else {
		if (fetchVarFromCfgFile((options->config == NULL)? DEFAULT_CONFIG : options->config,"port",&port,CFG_UINT,CFG_SIMPLE,0,NULL) == 0) {
			if (0 < port && port < 65536) {
				conf->port = (unsigned short) port;
			} else {
				fprintf(stderr,"illegal port in configuration: %i\n",port);
				return false;
			}
		} else {
			fprintf(stderr,"an error occured during configuration\n");
			return false;
		}
	}

	if (options->verbosity != 0) {
		conf->verbosity = options->verbosity;
	}

	if (options->user != NULL) {
		conf->user = strdup(options->user);
	}
	
	if (options->group != NULL) {
		conf->group = strdup(options->group);
	}

	conf->unlock();
		
	return true;
}

void conf_lock() {
	extern Conf conf;
	conf->mutex->lock(conf->mutex);
}

void conf_unlock() {
	extern Conf conf;
	conf->mutex->unlock(conf->mutex);
}
