/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>

#include "common.h"
#include "thread.h"
#include "log.h"
#include "packet.h"
#include "handler.h"

Handler get_handler(int type) {
	switch (type) {
		case HANDLER_PASSTHRU:
			return pthru_create();
		default:
			WARN("unknown handler type requested: %i",type);
			return NULL;	
	}
}


Handler pthru_create() {
	PthruHandler this;
	extern int pthru_init(Handler,void *,int);
	extern void pthru_destroy(Handler);
	
	if ((this = malloc(sizeof(pthru_handler_t))) == NULL) {
		WARN("could not create new passthru handler");
		return NULL;
	}
	bzero(this,sizeof(pthru_handler_t));

	this->handler.init = pthru_init;
	this->handler.free = pthru_destroy;
	
	return (Handler)this;
}

void pthru_destroy(Handler this) {
	close(this->sd);
	free(this);
}

int pthru_init(Handler this,void *buf,int len) {
	unsigned short int port;
	port = *(unsigned short int *)buf;
	buf += sizeof(unsigned short int);
	DEBUG("inititating passthrough handler");
}
