/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#ifndef _LIST_H
#define _LIST_H 1

#include "common.h"
#include "thread.h"


/*
 * List
 *
 */
struct list;
typedef struct list list_t;
typedef list_t *List;

struct list {
	/* Object API */
	Object object;
	void (*bind)(List);
	void (*release)(List);
	void (*lock)(List);
	void (*unlock)(List);
	/* end API */

	void **_list;
	size_t _min;
	size_t _size;
	size_t _count;

	size_t (*size)(List);
	size_t (*count)(List);
	bool (*grow)(List,size_t);
	bool (*shrink)(List,size_t);
	void *(*pop)(List);
	void *(*shift)(List);
	void *(*push)(List,void*);
	void *(*unshift)(List,void*);
};

List list_create(size_t size);
void list_destroy(List);
void list_destructor(void*);
void list_bind(List);
void list_release(List);
void list_lock(List);
void list_unlock(List);
size_t list_size(List);
size_t list_count(List);
bool list_grow(List,size_t);
bool list_shrink(List,size_t);
void *list_pop(List);
void *list_shift(List);
void *list_push(List,void*);
void *list_unshift(List,void*);

/* end List */

#endif /* _LIST_H */
