/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "common.h"
#include "options.h"
#include "conf.h"
#include "log.h"


void log_init() {
	openlog(PACKAGE,LOG_CONS | LOG_PERROR | LOG_PID,LOG_DAEMON);
}


void log(int type,char *file,int line,char *format,...) {
	va_list args;
	int level;
	char str[255];
	static const int prefix_len = 8; /* sprintf should write over NIL */
	
	switch(type) {
		case _ERROR_:
			level = LOG_ERR;
			strcpy(str,"error - ");
			break;
		case _WARN_:
			level = LOG_WARNING;
			strcpy(str,"warn  - ");
			break;
		case _INFO_:
			level = LOG_INFO;
			strcpy(str,"info  - ");
			break;
		case _DEBUG_:
			level = LOG_DEBUG;
			strcpy(str,"debug - ");
			break;
		default:
			syslog(LOG_ERR,"error - log: invalid argument specified, %i, in file %s, line %i",type,file,line);
			level = LOG_ERR;
			strcpy(str,"error - ");
	}

	va_start(args,format);
	vsnprintf(&str[prefix_len],245,format,args);
	va_end(args);
	
	syslog(level,str);
}
