/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>

#include "common.h"
#include "options.h"
#include "log.h"
#include "thread.h"
#include "conf.h"
#include "pdetach.h"
#include "server.h"
#include "prng.h"
#include "main.h"


int main(int argc, char *argv[]) {
	extern Options options;
	extern Conf conf;
	Server server;

	/* Options */ {
		options_init();

		unless (options_parse(argc,argv)) {
			fprintf(stderr,"%s: failed to parse options\n",*argv);
				return EXIT_FAILURE;
		}
	
		if (options->help) {
			usage();
			return EXIT_SUCCESS;
		} else if (options->license) {
			license();
			return EXIT_SUCCESS;
		} else if (options->version) {
			version();
			return EXIT_SUCCESS;
		}
	} /* end options */

	/* Configuration */ {
		conf_init();

		unless (conf_parse()) {
			fprintf(stderr,"%s - failed to parse configuration\n",PACKAGE);
				return EXIT_FAILURE;
		}
	} /* end Configuration */

	log_init();		/* init logging routines */
	rand_init();	/* init random data generator */

	INFO("anonnet started");
	
	unless (change_privileges()) {
		ERROR("there was a problem changing privileges ... better to quit here");
		INFO("anonnet exiting");
		return EXIT_FAILURE;
	}
		
	if (conf->daemon) {
		DEBUG("forking process....");
		unless (pdetach()) {
			ERROR("error forking");
			INFO("anonnet exiting");
			return EXIT_FAILURE;
		}
	}


	if ((server = server_create()) == NULL) {
		ERROR("could not get a server");
		INFO("anonnet exiting");
		return EXIT_FAILURE;
	}


	if (server->start(server) == false) {
		ERROR("could not start server");
		INFO("anonnet exiting");
		return EXIT_FAILURE;
	}

	while (true) {
		sleep(1);
		unless (server->isAlive(server)) {
			ERROR("server is not alive");
			INFO("anonnet exiting");
			return EXIT_FAILURE;
		}
	}

	INFO("anonnet exiting");
	return EXIT_SUCCESS;
}


bool change_privileges() {
	extern Conf conf;
	struct group *group;
	struct passwd *passwd;
		

	if (conf->group != NULL) {
		if ((group = getgrnam(conf->group)) != NULL || (group = getgrgid((gid_t)atoi(conf->group))) != NULL) {
			if (setgid(group->gr_gid) != 0) {
				WARN("could not change to group %s: %s",group->gr_name,strerror(errno));
				return false;
			} else {
				INFO("changed to group %s",group->gr_name);
			}
		} else {
			WARN("could not change to group %s: invalid group or ENOMEM",conf->group);
			return false;
		}
	}
	
	if (conf->user != NULL) {
		if ((passwd = getpwnam(conf->user)) != NULL || (passwd = getpwuid((uid_t)atoi(conf->user))) != NULL) {
			if (setuid(passwd->pw_uid) != 0) {
				WARN("could not change to user %s: %s",passwd->pw_name,strerror(errno));
				return false;
			} else {
				INFO("changed to user %s",passwd->pw_name);
			}
		} else {
			WARN("could not change to user %s: invalid user or ENOMEM",conf->user);
			return false;
		}
	}
	
	return true;
}


void usage() {
	printf("%s\n",USAGE);
}

void license() {
	printf("%s\n",LICENSE);
}

void version() {
	printf("%s %s\n",PACKAGE,VERSION);
}
