/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#ifndef _NODE_H
#define _NODE_H 1

#include <openssl/dh.h>

#ifndef _MCRYPT_H
#define _MCRYPT_H
#include <mcrypt.h>
#endif

#include "common.h"
#include "thread.h"
#include "packet.h"
#include "link.h"

struct node;
typedef struct node node_t;
typedef node_t *Node;

struct node {
	/* Object API */
	Object object;
	void (*bind)(Node);
	void (*release)(Node);
	void (*lock)(Node);
	void (*unlock)(Node);
	/* end API */

	Link inner;
	Link outer;

	Thread thread;
	bool (*start)(Node);
	bool (*isAlive)(Node);
	void (*setInner)(Node,int);
	void (*setHop)(Node,int);

	DH *dh;
	MCRYPT td;
	int hopid;
	unsigned char cipher_key[16];
	unsigned char hmac_key[16];
};

Node node_create(int);
void node_destroy(Node);
void node_destructor(void *);
void node_set_inner(Node,int);

bool node_start(Node);
bool node_is_alive(Node);
void *node_thread_entry(void *);
void node_run(Node);

void node_bind(Node);
void node_release(Node);
void node_lock(Node);
void node_unlock(Node);

bool node_link_init(Node);
bool node_node_init(Node);
void node_set_hop(Node,int);

#endif /* _NODE_H */
