/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include "common.h"
#include "options.h"

void options_init() {
	extern Options options;

	if ((options = malloc(sizeof(options_t))) == NULL) {
		perror("an error occured parsing options");
		exit(EXIT_FAILURE);		
	}
	memset(options,0,sizeof(options_t));
}

bool options_parse(int argc,char *argv[]) {
	extern Options options;
	int c;

	static struct option getopt_options[] = {
			{"config",     1, NULL, 'c'},
			{"port",       1, NULL, 'p'},
			{"daemon",     0, NULL, 'd'},
			{"foreground", 0, NULL, 'f'},
			{"user",       1, NULL, 'u'},
			{"group",      1, NULL, 'g'},
			{"tty",        0, NULL, 't'},
			{"verbose",    0, NULL, 'v'},
			{"help",       0, NULL, 1},
			{"license",    0, NULL, 2},
			{"version",    0, NULL, 3}
	};

	options->command    = *argv;
	options->config     = NULL;
	options->port       = 0;
	options->daemon     = false;
	options->foreground = false;
	options->user       = NULL;
	options->group      = NULL;
	options->usetty     = false;
	options->tty        = NULL;
	options->verbosity  = 0;
	options->help       = false;
	options->license    = false;
	options->version    = false;

	while ((c = getopt_long(argc,argv,"+c:p:dfu:g:tv",getopt_options,NULL)) != -1) {
		switch (c) {
			case 'c':
				if ((options->config = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options->command);
					perror("an error occured parsing options");
					options_destroy(options);
					return false;
				}
				break;
			case 'p':
				options->port = (unsigned short)atoi(optarg);
				break;
			case 'd':
				options->daemon = true;
				break;
			case 'f':
				options->foreground = true;
				break;
			case 'u':
				if ((options->user = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options->command);
					perror("an error occured parsing options");
					options_destroy(options);
					return false;
				}
				break;
			case 'g':
				if ((options->group = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options->command);
					perror("an error occured parsing options");
					options_destroy(options);
					return false;
				}
				break;
			case 't':
				options->usetty = true;
				if (optarg != NULL) {
					if ((options->tty = (char *)strdup(optarg)) == NULL) {
						fprintf(stderr,"%s: ",options->command);
						perror("an error occured parsing options");
						options_destroy(options);
						return false;
					}
					fprintf(stderr,"%s: cannot take a tty argument yet ... using current terminal instead\n",options->command);
				}
				break;
			case 'v':
				options->verbosity++;
				break;
			case 1:
				options->help = true;
				break;
			case 2:
				options->license = true;
				break;
			case 3:
				options->version = true;
				break;
			case '?':
			default:
				/* getopt prints to stderr */
				exit(EXIT_FAILURE);
		}

	}

	if ((options->daemon) && (options->foreground)) {
		fprintf(stderr,"%s: cannot use foreground and daemon switches together\n",*argv);
		exit(EXIT_FAILURE);
	}
		
	return true;
}

void options_destroy(Options options) {
	free(options->config);
	free(options->user);
	free(options->group);
	free(options->tty);
	free(options);
}

