/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#ifndef _PACKET_H
#define _PACKET_H 1

#include <openssl/dh.h>
#include "common.h"


#define NODE_MAX_P_SIZE 128
#define NODE_MAX_G_SIZE 128
#define NODE_MAX_PUBK_SIZE 128
#define NODE_MAX_SEED_SIZE 128
#define NODE_INIT_INPUT_SIZE (4+(4*128))


#define FLAG_CLR(f) ((f) = 0)
#define FLAG_SET(f,b) ((f) |= (b))
#define FLAG_ISSET(f,b) ((f) & (b))


#define PACKET_SIZE 129
#define PACKET_DATA_SIZE 48
#define PACKET_HEAD_SIZE 16


/*	Link Packet Flags
 *
 *	00 - NULL
 *	01 - NBND
 *	02 - INIT
 *	04 - RSRV
 *	08 - RSRV
 *	10 - RSRV
 *	20 - RSRV
 *	40 - RSRV
 *	80 - RSRV
 *	FF - KILL
 */
#define PACKET_LINK_NULL 0x00
#define PACKET_LINK_NBND 0x01
#define PACKET_LINK_INIT 0x02
#define PACKET_LINK_KILL 0xFF

/*	Virtual Link Packet Flags
 *
 *	00 - NULL
 *	01 - FWRD - Forward
 *	02 - INIT - Initiation
 *	04 - CNCT - Connect to New
 *	08 - HNDL - Invoke Handler
 *	10 - RSRV - Reserved
 *	20 - RSRV - Reserved
 *	40 - RSRV - Reserved
 *	80 - RSRV - Reserved
 *	FF - KILL - Kill
 */
#define PACKET_NODE_NULL 0x00
#define PACKET_NODE_FWRD 0x01
#define PACKET_NODE_INIT 0x02
#define PACKET_NODE_CNCT 0x04
#define PACKET_NODE_HNDL 0x08
#define PACKET_NODE_KILL 0xFF

#define NODE_MAC_POS(hop)  (((hop)*16) + 1)
#define NODE_SEQ_POS(hop)  (((hop)*16) + 11)
#define NODE_FLAG_POS(hop) (((hop)*16) + 15) /* relative to Link buffer */

#define NODE_ENC_BEG(hop) (((hop)*16) + 1)
#define NODE_ENC_END(hop) 129
#define NODE_ENC_LEN(hop) (NODE_ENC_END(hop) - NODE_ENC_BEG(hop))

#define NODE_HASH_BEG(hop) (((hop)*16) + 11)
#define NODE_HASH_END(hop) 129
#define NODE_HASH_LEN(hop) (NODE_HASH_END(hop) - NODE_HASH_BEG(hop))
#define NODE_DATA_POS(hop) 81 /* relative to Link buffer */
#define PACKET_BOUNDARY(n) ((n%PACKET_DATA_SIZE > 0)? (n+(PACKET_DATA_SIZE-(n%PACKET_DATA_SIZE))) : n)

#endif /* _PACKET_H */
