/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#include <netinet/in.h>

#include <sys/types.h>
#include <sys/socket.h>

#include "common.h"
#include "thread.h"
#include "log.h"
#include "conf.h"
#include "node.h"
#include "server.h"


server_t *server_create(void (*call_back)(int),int port) {
	server_t *server;

	if ((server = malloc(sizeof(server_t))) == NULL) {
		ERROR("could not malloc space for a new server");
		return NULL;	
	}
	memset(server,0,sizeof(server_t));

	object_init((object_t *)server,__server_free,server);
	thread_init((thread_t *)server,__server_run,server);

	server->call_back	= call_back;
	server->port		= port;

	return server;
}

void server_free(server_t *this) {

	thread_destroy((thread_t *)this);
	object_destroy((object_t *)this);

	free(this);
}


void __server_free(void *this) {
	server_free((server_t *)this);
}


void __server_run(void *arg) {
	server_run((server_t *)arg);

	return; /* void */
}

void server_run(server_t *server) {
	int s, sd;
	struct sockaddr_in sin;
	const socklen_t size = (socklen_t)sizeof(struct sockaddr_in);
	short port;
	node_t *node;
	
	port = server->port;

	memset(&sin, 0, sizeof(size));
	sin.sin_family      = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port        = htons(port);


	if ((s = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP)) < 0) { /* IP or TCP? */
		ERROR("could not create socket: %s",strerror(errno));
		goto server_end;
	}

	if (bind(s, (struct sockaddr *)&sin,size) < 0) {
		ERROR("could not bind to socket: %s",strerror(errno));
		goto server_end;
	}
	
	for (;;) {
	
		if (listen(s,SOMAXCONN) < 0) {
			ERROR("could not listen to socket: %s",strerror(errno));
			goto server_end;
		}

		DEBUG("waiting for connection");

		if ((sd = accept(s,NULL,0)) < 0 ) {
			ERROR("could not accept connection on socket: %s",strerror(errno));
			goto server_end;
		}

		DEBUG("accepted connection");

		DEBUG("calling call-back function");
		server->call_back(sd);

	}

	server_end:

	INFO("server finishing");

	return;
}

